
/*
*  Javascript from template: browse
*/ 
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * The jira_controller object
 * This is used to wire up the view and model with actions
 */ 
var jira_browse_controller = {
    init:function() {
        //
        // If there is no URL for searching, don't show 
        // the search panel
        //
        var query = "";
        var searchUrl = '"https://issues.apache.org:443/jira/rest/api/latest/project/"+query.toUpperCase() +"/versions&username="+jira_browse_controller.username+"&password="+jira_browse_controller.password'; 
        if(searchUrl === "") $("#searchPanel").hide();
        jira_browse_controller.update();
        //
        // If there is a "requiresLogin" attribute, don't
        // trigger search just yet
        //
        if(!jira_browse_controller.requiresLogin)
        jira_browse_controller.search()
    },
    
    /**
     * Search the remote service and sort results if possible.
     */
    search:function(query, sort, order) {
	if (sort === undefined) {
	    sort = "updated";
	}
    	if (query === undefined || query == "") {
	    var url = widget.proxify("https://issues.apache.org:443/jira/rest/api/latest/project/WOOKIE/versions&username="+jira_browse_controller.username+"&password="+jira_browse_controller.password);
    	} else {
    	    var url = widget.proxify("https://issues.apache.org:443/jira/rest/api/latest/project/"+query.toUpperCase() +"/versions&username="+jira_browse_controller.username+"&password="+jira_browse_controller.password);
    	}
    	jira_browse_controller.populate(url);
    },
	
	
    /**
     * Extract the values from the search form, do the search and display the results.
     */
    submitSearchForm:function() {
	var query = $('#searchString').val();
	var sort = $('#sort').val();
	var order = ($("input[@name='sortOrder']:checked").val());
	jira_browse_controller.search(query, sort, order);
    },

    /**
     * Update the display
     */
    update:function() {
	// expand the search box on screens that are either wide enough (alongside results) or tall enough
	if ($.mobile.media("only screen and (min-width : 801px)")) {
	    $('#searchPanel').trigger('expand');
	} else if ($.mobile.media("only screen and (min-width : 641px) and (max-width : 1024px)") || $.mobile.media("only screen and (min-width : 1801px)")) {
	    $('#searchPanel').trigger('expand');
	} else {
	    $('#searchPanel').trigger('collapse');
	}
    },

   /**
    * Get the current title for this widget.
    */
    getTitle:function() {
        return "Jira";
    },

   /**
    * Populate the results list with data from a given URL.
    */
    populate:function(sourceUrl) {
        $('#results').remove();
        var html = jira_browse_controller.transform(sourceUrl, "list");      
        $('#content-primary').html(html).trigger("create");
        $('body').trigger("results_updated");
	    $('.result:first').trigger('expand');
	    jira_browse_controller.update();
    },
        
    /**
     * Retrieve the details of an item and display them in the detail section.
     */
    displaySummary:function(itemId){
    
        /**
         * Locate the detail element to update
         */
        var detailElement = $(".result[wid='"+itemId+"']").find(".detail");
        
        /**
         * Load the data, showing a loading message while waiting
         */
        detailElement.html("<p>Loading...</p>");
        var sourceUrl = widget.proxify("https://issues.apache.org:443/jira/rest/api/latest/version/" + itemId + "/unresolvedIssueCount&username="+jira_browse_controller.username+"&password="+jira_browse_controller.password);
	    $.mobile.showPageLoadingMsg();
        var html = jira_browse_controller.transform(sourceUrl, "detail");
        detailElement.html(html);    
        
        /**
         * Add events
         */ 
	    detailElement.click(function() {
	    var event = { widget: "jira", type: "clickItem", itemId: itemId};	    
	    jira_controller.executeCallbacks(event);
	});
	$.mobile.hidePageLoadingMsg();
    }			  
}

/**
 * Initialise page layout
 */
$('#home').live('pageinit',function(event) {
   jira_browse_controller.init(); 
});


/**
 * Display the content of a result item when it is expanded.
 */
$('div.result').live('expand', function(event) {
    var wid = $(this).attr("wid");
    jira_browse_controller.displaySummary(wid);
    var itemId = wid.substr(wid.indexOf("id=")+3);
    var event = { widget: "jira", type: "expandItem", itemId: itemId};	    
    jira_controller.executeCallbacks(event );
});


/**
 * load XML data and transform it into HTML
 * @param src the URL to load
 * @param type the type of data transform requested, either "detail" or "list"
 */
jira_browse_controller.transform = function(src, type){
 var output = "";
 $.support.cors = true; // force cross-site scripting (as of jQuery 1.5)
 $.ajax({
  url: src,
  dataType: "json",
  async: false,
  success: function(xml){
    output = jira_browse_controller.transformXml(xml, type); 
  }, //end success function
    error: function(xhr, textStatus, errorThrown) {
        alert(errorThrown);
    }
 });
 
 return output;
}

/**
 * Transform data into HTML
 */ 
jira_browse_controller.transformXml = function(xml, type){

    if (type === "detail"){
      template = "<!--  Licensed to the Apache Software Foundation (ASF) under one or more  contributor license agreements.  See the NOTICE file distributed with  this work for additional information regarding copyright ownership.  The ASF licenses this file to You under the Apache License, Version 2.0  (the 'License'); you may not use this file except in compliance with  the License.  You may obtain a copy of the License at      http://www.apache.org/licenses/LICENSE-2.0  Unless required by applicable law or agreed to in writing, software  distributed under the License is distributed on an 'AS IS' BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the specific language governing permissions and  limitations under the License.-->    <p>Issues to resolve: ${ISSUESUNRESOLVEDCOUNT}</p>     ";
    } else {
      template = "<!--  Licensed to the Apache Software Foundation (ASF) under one or more  contributor license agreements.  See the NOTICE file distributed with  this work for additional information regarding copyright ownership.  The ASF licenses this file to You under the Apache License, Version 2.0  (the 'License'); you may not use this file except in compliance with  the License.  You may obtain a copy of the License at      http://www.apache.org/licenses/LICENSE-2.0  Unless required by applicable law or agreed to in writing, software  distributed under the License is distributed on an 'AS IS' BASIS,  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the specific language governing permissions and  limitations under the License.--><div class='result' data-role='collapsible' wid='${ID}'>  <h3>${NAME} <span id='${ID}-released'></span> </h3>  <p>${DESCRIPTION}</p>  <div class='detail'>  </div>   <script>      if (${RELEASED})          $('#${ID}-released').html('<em>Released: ${USERRELEASEDATE}</em>');   </script></div>";
    }
    
    var output = "";
    var items = "";
    
    /**
     * For each element that matches itemName, create a new ItemTemplate and
     * replace placeholders in the template with values from the XML using the ItemElements 
     * and ItemAttrobutes lists
     */
    var elements = jira_browse_controller.find(xml,"");
    $(elements).each(
       function(){
         var item = template;
         var elements = "".split(",");
         for (var i=0;i<elements.length;i++){
            var element = elements[i]; 
            var pattern = "\\$\{" + element.toUpperCase() +"\}";
            item = item.replace(new RegExp(pattern,'g'), $(this).find(element).first().text());
         }
         var attributes = "id,description,name,userReleaseDate,released,issuesUnresolvedCount".split(",");
         for (var i=0;i<attributes.length;i++){
            var attribute = attributes[i]; 
            var pattern = '\\$\{'+attribute.toUpperCase()+'\}';
            item = item.replace(new RegExp(pattern,'g'), $(this).attr(attribute));
         }
         items += item;
       }
    );
    
    /**
     * If we are returning a list, wrap the result in the collection template, otherwise
     * just return the item
     */
    if (type === "list"){
       output = "<div id='results' data-role='collapsible-set'>${ITEMS}</div>".replace("${ITEMS}", items);
    } else {
        output = items;
    }
    return output;
}

/**
 * Obtain collection of objects to operate on, either
 * XML elements or JSON objects
 */ 
jira_browse_controller.find = function(data, name){
  if ("json"==="xml"){
    return $(data).find(name);
  } else {
    if (!name || name === "") return data;
    return data[name];
  }
}
/*
*  Javascript from template: base
*/ 
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * The @widget.shortname@_controller object
 * This is used to wire up the view and model with actions
 */ 
var jira_controller = {
    /**
     * A dictionary of callback functions that are called whenever an
     * event is fired by this widget.
     */
    callbacks: {},

    init:function() {
        jira_controller.update();
    },

    /**
     * Update the display
     */
    update:function() { 
    },

   /**
    * Get the current title for this widget.
    */
    getTitle:function() {
        return "Jira";
    },

    /**
     * Get the viewport width and height.
     * returns an object with width and height properties.
     */
    getViewport:function() {
	var e = window
	, a = 'inner';
	if ( !( 'innerWidth' in window ) ) {
	    a = 'client';
	    e = document.documentElement || document.body;
	}
	return { width : e[ a + 'Width' ] , height : e[ a + 'Height' ] }
    },

    /**
     * Register a callback function. 
     *
     * type: the name of the type of event to respond to 
     *
     * widget: [optional] the name of the widget where the event must
     * occur for the callback to be executed. This allows callbacks to
     * be register so that they will only respond to events in
     * specific widgets, or to all events.
     */
    register:function(event, callback, widget) {
	if (widget === undefined) {
	    var cbks = jira_controller.callbacks[event];
	} else {
	    var cbks = jira_controller.callbacks[widget + "." + event];
	}
	if (cbks === undefined) {
	    cbks = new Array();
	}
	cbks.push(callback);
	jira_controller.callbacks[event] = cbks;
    },

    /**
     * Execute all the callbacks registered for a given event.
     *
     * The event object is passed directly to the callback function
     * and can contani any number of properties. Two properties that
     * important to the callback functionality are:
     *
     * type: the name of the type of event to respond to 
     *
     * widget: [optional] the name of the widget where the event
     * occured. This allows callbacks to be register so that they will
     * only respond to events in specific widgets, or to all events.
     */
    executeCallbacks:function(event) {
	// Execute all callbacks not restricted to a widget
	var cbks = jira_controller.callbacks[event.type];
	if (cbks === undefined) return;
	for (var i = 0; i < cbks.length; i++) {
	    cbks[i](event);
	}

	// Execute all callbacks restricted to a widget
	if (event.widget === undefined) return;
	var cbks = jira_controller.callbacks[event.widget + "." + event.type];
	if (cbks === undefined) return;
	for (var i = 0; i < cbks.length; i++) {
	    cbks[i](event);
	}
    }
};

/**
 * Provides a basic shim for opening the widget without a widget object (e.g. directly in browsers)
 */
if (!window.widget){
    window.widget = {};
}

/**
 * Adds in the "proxify" method if it isn't in the widget object, e.g. as we're opening the widget
 * directly in a browser, or using a widget runtime other than Wookie e.g. Opera, PhoneGap etc
 */
if (!window.widget.proxify){
    window.widget.proxify = function(url){ return url };
}

$('#home').live('pageshow',function(event) {
   jira_controller.init(); 
});

$('body').bind('orientationchange',function(event){
    jira_controller.update();
})
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
jira_browse_controller.requiresLogin=true;

//
// Hide the search panel until logged in
//
$('#home').live( 'pageinit',function(event){
    $('#searchPanel').hide(); 
});

/**
 * Login action
 *
 * Sets the username and password to use during this browser session
 * and toggles the login panel and search panel
 */
jira_browse_controller.submitLoginForm = function(){

    $.mobile.showPageLoadingMsg();
    jira_browse_controller.username = $('#usernameString').val();
    jira_browse_controller.password = $('#passwordString').val();
    
    //
    // Check if the credentials can pull down session info; if so they are
    // correct and we can use them. Note we don't actually create a session, we
    // use Basic Auth for each request, and this is just to check credential
    // validity
    //
    // If the credentials are OK, we hide the login form and show the search
    // form.
    //
    $.ajax({
      url: window.widget.proxify('https://issues.apache.org:443/jira/rest/auth/1/session')+"&username="+ jira_browse_controller.username + "&password="+jira_browse_controller.password,
      dataType: 'json',
      success: function(data){
        $.mobile.hidePageLoadingMsg();
        jira_browse_controller.requiresLogin=false;
        $('#loginPanel').hide();
        $('#searchPanel').trigger('expand');  
        $('#searchPanel').show(); 
      },
      
      error: function(){
        $.mobile.hidePageLoadingMsg();
        $('#loginMessage').text("login rejected, please try again");
      }
    });
    
    
}
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
jira_browse_controller.requiresLogin=true;

//
// Hide the search panel until logged in
//
$('#home').live( 'pageinit',function(event){
    $('#searchPanel').hide(); 
});

/**
 * Login action
 *
 * Sets the username and password to use during this browser session
 * and toggles the login panel and search panel
 */
jira_browse_controller.submitLoginForm = function(){

    $.mobile.showPageLoadingMsg();
    jira_browse_controller.username = $('#usernameString').val();
    jira_browse_controller.password = $('#passwordString').val();
    
    //
    // Check if the credentials can pull down session info; if so they are
    // correct and we can use them. Note we don't actually create a session, we
    // use Basic Auth for each request, and this is just to check credential
    // validity
    //
    // If the credentials are OK, we hide the login form and show the search
    // form.
    //
    $.ajax({
      url: window.widget.proxify('https://issues.apache.org:443/jira/rest/auth/1/session')+"&username="+ jira_browse_controller.username + "&password="+jira_browse_controller.password,
      dataType: 'json',
      success: function(data){
        $.mobile.hidePageLoadingMsg();
        jira_browse_controller.requiresLogin=false;
        $('#loginPanel').hide();
        $('#searchPanel').trigger('expand');  
        $('#searchPanel').show(); 
      },
      
      error: function(){
        $.mobile.hidePageLoadingMsg();
        $('#loginMessage').text("login rejected, please try again");
      }
    });
    
    
}
